import argparse
import logging


import assetadapter.common as fa_common
import rpc.client as client


def main():
    # Parse command line arguments.
    parser = argparse.ArgumentParser()
    parser.add_argument("-H", "--host", help="Automation server host of field agent.", type=str, default="localhost")
    parser.add_argument("-a", "--alias", help="Alias of the device attended to control.", type=str)
    parser.add_argument("-e", "--exchange", help="RabbitMQ exchange for RPC.", type=str, default='visa_rpc')
    parser.add_argument("-u", "--username", help="RabbitMQ username.", type=str, default='guest')
    parser.add_argument("-p", "--password", help="RabbitMQ password.", type=str, default='guest')
    parser.add_argument("-P", "--port", help="RabbitMQ port.", type=int, default=5672)
    parser.add_argument("-A", "--address", help="Address of the device attended to control.", type=str)
    parser.add_argument("-c", "--command", help="Command transmitted to the device.", type=str, default="?IDN")
    parser.add_argument("-v", "--verbose", help="Increase output verbosity.", action="store_true")
    args = parser.parse_args()

    # Setup logging
    if args.verbose:
        root = fa_common.setupLogging(logging.DEBUG)
    else:
        root = fa_common.setupLogging(logging.INFO)

    root.info('Start Field Agent CLI')
    root.info('Host: ' + args.host)
    root.info('Device: ' + args.alias + ' on ' + args.address)
    root.info('Connect to Field Agent ...')

    rpcClient = client.VisaRpcClient(exchange=args.exchange, host=args.host, port=args.port, username=args.username, password=args.password)
    result = {}
    result[args.alias] = args.address
    rpcClient.alias(result)
    print(args.alias + ':' + args.command + ' -> ' + str(rpcClient.query({
        u'alias': args.alias,
        u'command': args.command
    })))
    rpcClient.dispose()
    root.info('Connection to Field Agent closed.')



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-11 21:25
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('visaweb', '0012_auto_20170311_2124'),
    ]

    operations = [
        migrations.AlterField(
            model_name='visadevice',
            name='alias',
            field=models.CharField(default='1a357258-ae90-48ff-af86-c97934780e93', max_length=200),
        ),
    ]



import json
import logging
import pika

import assetadapter.devicemanager as dm
import rpc

import schedule
import time
import datetime

import platform

schedule.run_continuously(interval=1)


def globalTick():
    print('Synchronization begins ...')

    print('Synchronization ended.')


# schedule.every(10).seconds.do(globalTick)

class VisaRpcServer(rpc.RpcServer):
    def __init__(self, visa_library, username, password, name='visa_rpc_server', exchange="visa_rpc",
                 host='localhost'):
        rpc.RpcServer.__init__(self, name=name, host=host, exchange=exchange, username=username, password=password)
        self.device_manager = dm.VisaDeviceManager(visa_library=visa_library)

        self.create_method(name='alias', request_handler_method=self.on_alias_request)
        logging.info('- Registrate consumer method LOCAL.alias')

        self.create_method(name='query', request_handler_method=self.on_query_request)
        logging.info('- Registrate consumer method LOCAL.query')

        self.create_method(name='read', request_handler_method=self.on_read_request)
        logging.info('- Registrate consumer method LOCAL.read')

        self.create_method(name='write', request_handler_method=self.on_write_request)
        logging.info('- Registrate consumer method LOCAL.write')

        self.create_method(name='list', request_handler_method=self.on_list_request)
        logging.info('- Registrate consumer method LOCAL.list')

        self.create_method(name='execution', request_handler_method=self.on_execute)
        logging.info('- Registrate consumer method LOCAL.exec')

        self.create_method(name='close', request_handler_method=self.on_close_channel)
        logging.info('- Registrate consumer method LOCAL.close_channel')

        self.data_channels = []
        self.synchInterval = 3

    def on_list_request(self, ch, method, props, body):
        response = []

        try:
            request = json.loads(body.decode("utf-8"))
            response = self.device_manager.list(request)
            logging.info('LOCAL.list: ' + json.dumps(request) + ' -> ' + json.dumps(response))

        except dm.ScVisaError as ex:
            logging.warning('Exception happened on LOCAL.list ' + str(ex))
            self.send_event(body={
                'error': ex,
                'request': body
            })
        except Exception as e:
            logging.error(e)
        finally:
            self.return_acknowledgement(ch=ch, method=method, props=props, response=response)

    def on_alias_request(self, ch, method, props, body):
        response = {}
        try:
            request = json.loads(body.decode("utf-8"))
            response = self.device_manager.alias(request)
            logging.info('LOCAL.alias: ' + json.dumps(request) + ' -> ' + json.dumps(response))
        except dm.ScVisaError as ex:
            logging.warning('Exception happened on LOCAL.alias ' + str(ex))
            self.send_event(body={
                'error': ex,
                'request': body
            })

        except Exception as e:
            logging.error(e)
        finally:
            self.return_acknowledgement(ch=ch, method=method, props=props, response=response)

    def on_query_request(self, ch, method, props, body):
        response = {
            'result': None
        }
        try:
            request = json.loads(body.decode("utf-8"))
            response = self.device_manager.query(request)
            logging.info('LOCAL.query: ' + json.dumps(request) + ' -> ' + json.dumps(response)[:20] + ' ...')

        except dm.ScVisaError as ex:
            logging.warning('Exception happened on LOCAL.query ' + str(ex))
            self.send_event(body={
                'error': ex,
                'request': body
            })
        except Exception as e:
            logging.error(e)
        finally:
            self.return_acknowledgement(ch=ch, method=method, props=props, response=response, headers={
                '__TypeId__': 'hu.sagax.hedgehog.visa.M2mResponse'
            })

    def on_write_request(self, ch, method, props, body):
        response = {}
        try:
            request = json.loads(body.decode("utf-8"))
            response = self.device_manager.write(request)
            logging.info('LOCAL.write: ' + json.dumps(request) + ' -> ' + json.dumps(response)[:20] + ' ...')

        except dm.ScVisaError as ex:
            logging.warning('Exception happened on LOCAL.write ' + str(ex))
            self.send_event(body={
                'error': ex,
                'request': body
            })
        except Exception as e:
            logging.warning('Exception happened on LOCAL.write ...')
            logging.error(e)
        finally:
            self.return_acknowledgement(ch=ch, method=method, props=props, response=response)

    def on_read_request(self, ch, method, props, body):
        response = {
            'result': None
        }
        try:
            request = json.loads(body.decode("utf-8"))
            response = self.device_manager.read(request)
            logging.info('LOCAL.read: ' + json.dumps(request) + ' -> ' + json.dumps(response)[:20] + ' ...')

        except dm.ScVisaError as ex:
            logging.warning('Exception happened on LOCAL.read ' + str(ex))
            self.send_event(body={
                'error': ex,
                'request': body
            })
        except Exception as e:
            logging.error(e)
        finally:
            self.return_acknowledgement(ch=ch, method=method, props=props, response=response, headers={
                '__TypeId__': 'hu.sagax.hedgehog.visa.M2mResponse'
            })

    def on_execute(self, channel, method, props, body):
        request = json.loads(body.decode("utf-8"))
        response = {}
        # for command in request['setup']:
        try:
            self.data_channels.append(request)
            logging.info('LOCAL.execute: ' + json.dumps(
                request) + ' opened new channel with sampling in ' + str(self.synchInterval) + ' ms.')

            response = {
                'executed': True,
                'sampling': self.synchInterval,
                'domain': request['domain'],
                'controller': platform.node()
            }

            schedule.every(self.synchInterval).seconds.do(self.tick_job, request['alias'], request['domain'],
                                                          request['command']).tag(request['domain'])

        except dm.ScVisaError as ex:
            logging.warning('Exception happened on LOCAL.execute ' + str(ex))
            self.send_event(body={
                'error': ex,
                'request': body
            })

            response = {
                'executed': False
            }
        except Exception as e:

            response = {
                'executed': False
            }
            logging.error(e)
        finally:
            self.return_acknowledgement(ch=channel, method=method, props=props, response=response, headers={
                '__TypeId__': 'hu.sagax.hedgehog.visa.M2mStreamResponse'
            })

    def on_close_channel(self, channel, method, props, body):
        request = json.loads(body.decode("utf-8"))
        response = {}
        # for command in request['setup']:
        try:
            self.data_channels.append(request)
            logging.info('LOCAL.close: ' + json.dumps(request) + ' close.')

            schedule.clear(request['domain'])

            response={
                'executed': True,
                'domain': request['domain'],
                'controller': platform.node()
            }
        except Exception as e:

            response={
                'executed': False
            }

            logging.error(e)
            self.send_event(body={
                'error': e,
                'request': body
            })
        finally:
            self.return_acknowledgement(ch=channel, method=method, props=props, response=response, headers={
                '__TypeId__': 'hu.sagax.hedgehog.visa.M2mStreamResponse'
            })

    def tick_job(self, alias, domain, command):
        logging.info(
            'TICK ' + alias + ':' + domain + '------------------------------------------------------------------')
        # for ch in self.data_channels:
        response = {}
        try:
            response = self.device_manager.visa_query(alias=alias, command=command).tolist()
            logging.info('LOCAL.query: ' + command + ' -> ' + json.dumps(response)[:20] + ' ...')
            self.send_data(alias=alias, domain=domain, timestamp=datetime.datetime.now(), value=response)
        except dm.ScVisaError as ex:
            logging.warning('Exception happened on tick query ' + str(ex))
            self.send_event(body={
                'error': ex,
                'request': response
            })
        except Exception as ex:
            logging.error(ex)
            self.send_event(body={
                'error': ex
            })

    def send_data(self, alias, domain, value, timestamp):
        message_properties = pika.BasicProperties(content_type='application/json', content_encoding='utf-8', headers={
            'alias': alias,
            'domain': domain,
            'controller': platform.node()
        })
        self.channel.basic_publish(
            exchange=self.exchange,
            routing_key='ingestion',
            properties=message_properties,
            body=json.dumps(obj={
                'timestamp': str(timestamp),
                'value': value
            }, ensure_ascii=False))
        logging.debug('Call \"' + "ingestion" + '\" request ' + str(value) + ' -> ' + str(message_properties))

    def send_event(self, body):
        message_properties = pika.BasicProperties(content_type='application/json', content_encoding='utf-8', headers={
            'controller': platform.node()
        })
        self.channel.basic_publish(
            exchange=self.exchange,
            routing_key='log',
            properties=message_properties,
            body=json.dumps(body))
        logging.debug('Call \"' + "ingestion" + '\" request ' + str(body) + ' -> ' + str(message_properties))



from django.conf.urls import url

from . import views

app_name = 'visaweb'
urlpatterns = [
    # ex: /visaweb/
    url(r'^$', views.IndexView.as_view(), name='index'),
    # ex: /visaweb/agilent_hp_8920b/
    url(r'^(?P<slug>[-\w]+)/$', views.DetailView.as_view(), name='detail'),
    # ex: /visaweb/agilent_hp_8920b/results/1/
    url(r'^results/(?P<pk>[0-9]+)/$', views.ResultsView.as_view(), name='results'),

    # # ex: /visaweb/
    # url(r'^$', views.index, name='index'),
    # # ex: /visaweb/agilent_hp_8920b/
    # url(r'^(?P<device_alias>\w+)/$', views.detail, name='detail'),
    # # ex: /visaweb/agilent_hp_8920b/results/1/
    # url(r'^(?P<device_alias>\w+)/results/(?P<event_id>[0-9]+)/$', views.results, name='results'),

    # ex: /visaweb/agilent_hp_8920b/spectrum/
    url(r'^(?P<device_alias>\w+)/spectrum/$', views.spectrum, name='spectrum'),
    # ex: /visaweb/agilent_hp_8920b/waterfall/
    url(r'^(?P<device_alias>\w+)/waterfall/$', views.waterfall, name='waterfall'),

    url(r'^/attach/$', views.attach, name='attach'),
    url(r'^(?P<device_alias>\w+)/read/$', views.read, name='read'),
    url(r'^(?P<device_alias>\w+)/write/$', views.write, name='write'),
    url(r'^(?P<device_alias>\w+)/query/$', views.query, name='query'),

]


import numpy
import scipy.fftpack

import matplotlib
matplotlib.use('Agg')
from matplotlib import pyplot
import mpld3
from mpld3 import plugins

class Spectrum:

    def __init__(self, properties):
        self.properties = properties

    def show(self, data):
        # fig, ax = plt.subplots()

        y = numpy.fromstring(data['result'], dtype=float, sep=",")
        x = numpy.linspace(
            start=self.properties['frequency_center'] - self.properties['frequency_span'],
            stop=self.properties['frequency_center'] + self.properties['frequency_span'],
            num=y.size)
        #y = np.sin(50.0 * 2.0 * np.pi * x) + 0.5 * np.sin(80.0 * 2.0 * np.pi * x)
        # yf = scipy.fftpack.fft(y)
        # xf = numpy.linspace(0.0, 1.0 / (2.0 * T), N / 2)

        fig, ax = pyplot.subplots()
        # ax.plot(x, 2.0 / N * numpy.abs(y[:N // 2]))
        ax.plot(x, y)

        # plt.show()


        # x = np.linspace(-2, 2, 20)
        # y = x[:, None]
        #
        # X = np.zeros((20, 20, 4))
        # X[:, :, 0] = np.exp(- (x - 1) ** 2 - (y) ** 2)
        # X[:, :, 1] = np.exp(- (x + 0.71) ** 2 - (y - 0.71) ** 2)
        # X[:, :, 2] = np.exp(- (x + 0.71) ** 2 - (y + 0.71) ** 2)
        # X[:, :, 3] = np.exp(-0.25 * (x ** 2 + y ** 2))
        #
        # im = ax.imshow(X, extent=(10, 20, 10, 20), origin='lower', zorder=1, interpolation='nearest')
        # fig.colorbar(im, ax=ax)
        #
        ax.set_title(self.properties['title'], size=20)

        #
        # # plugins.MousePosition(fontsize=14)
        # plugins.connect(fig=fig)

        return  mpld3.fig_to_html(fig=fig)

class Heatmap:

    def __init__(self, properties):
        self.properties = properties

    def show(self, data):
        fig, ax = pyplot.subplots()
        #
        # y = numpy.fromstring(data['result'], dtype=float, sep=",")
        # x = numpy.linspace(
        #     start=self.properties['frequency_center'] - self.properties['frequency_span'],
        #     stop=self.properties['frequency_center'] + self.properties['frequency_span'],
        #     num=y.size)
        # # y = np.sin(50.0 * 2.0 * np.pi * x) + 0.5 * np.sin(80.0 * 2.0 * np.pi * x)
        # yf = scipy.fftpack.fft(y)
        # xf = numpy.linspace(0.0, 1.0 / (2.0 * 10), 401 / 2)
        #
        # fig, ax = pyplot.subplots()
        # ax.plot(x, 2.0 / 401 * numpy.abs(y[:401 // 2]))
        # # ax.plot(x, y)
        #
        # # plt.show()


        x = numpy.linspace(-2, 2, 20)
        y = x[:, None]

        # X = numpy.zeros((20, 20, 4))
        y = numpy.fromstring(data['result'], dtype=float, sep=",")
        X = numpy.asmatrix(y)

        im = ax.imshow(X, extent=(10, 20, 10, 20), origin='lower', zorder=1, interpolation='nearest')
        fig.colorbar(im, ax=ax)

        ax.set_title(self.properties['title'], size=20)

        #
        plugins.MousePosition(fontsize=14)
        plugins.connect(fig=fig)

        return  mpld3.fig_to_html(fig=fig)


# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-11 21:16
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('visaweb', '0010_auto_20170311_2115'),
    ]

    operations = [
        migrations.AlterField(
            model_name='visadevice',
            name='alias',
            field=models.CharField(default='66bcf3f8-7f53-47c6-8eb0-3efb9283390e', max_length=200),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-11 16:58
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('visaweb', '0005_auto_20170311_1658'),
    ]

    operations = [
        migrations.AlterField(
            model_name='visadevice',
            name='alias',
            field=models.CharField(default='0f5842de-6d7d-4c12-b25a-85eebed30e45', max_length=200),
        ),
    ]



import argparse
import logging
import sys

import assetadapter.common as core
import rpc.server as rpcserver

defaultMessageQueueConfiguration = {
    'mq': {
        'host': 'localhost',
        'port': 5672,
        'username': 'hedgehog',
        'password': 'qwe123',
        'exchange': 'visa_rpc'
    }
}

defaultPyVisaConfiguration = {
    'assetadapter': {
        'inventory': 'local.yml',
        # 'backend': '@py'
        'backend': 'sagax-env.yml@sim'
    }
}


def main():
    # Load RabbitMQ configuration.
    externalizedMessageQueueConfiguration = core.loadConfiguration("mq.yml", defaults=defaultMessageQueueConfiguration)
    # Load Visa configuration
    externalizedVisaConfiguration = core.loadConfiguration("visa.yml", defaults=defaultPyVisaConfiguration)

    # Parse command line arguments.
    parser = argparse.ArgumentParser()
    parser.add_argument("-H", "--host", help="Automation server host of field agent.", type=str, default='localhost')
    parser.add_argument("-V", "--assetadapter", help="PyVISA assetadapter backend.", type=str, default='@py')
    parser.add_argument("-e", "--exchange", help="RabbitMQ exchange for RPC.", type=str, default='')
    parser.add_argument("-u", "--username", help="RabbitMQ username.", type=str, default='guest')
    parser.add_argument("-P", "--port", help="RabbitMQ port.", type=int, default=5672)
    parser.add_argument("-p", "--password", help="RabbitMQ password.", type=str, default='guest')
    parser.add_argument("-v", "--verbose", help="Increase output verbosity.", action="store_true")
    parser.add_argument("-l", "--logfile", help="Logging to file.", type=str, default="~/.hedgehog/log/output.log")
    commandLineArguments = parser.parse_args()

    # Setup logging
    if commandLineArguments.verbose:
        root = core.setupLogging(logfile=commandLineArguments.logfile, logging_level=logging.DEBUG)
    else:
        root = core.setupLogging(logfile=commandLineArguments.logfile, logging_level=logging.INFO)

    # Assemble configuration
    # configuration = {
    #     **externalizedMessageQueueConfiguration,
    #     **externalizedVisaConfiguration
    # }

    # Initialize VISA RPC server
    server = rpcserver.VisaRpcServer(
        host=externalizedMessageQueueConfiguration['mq']['host'],
        username=externalizedMessageQueueConfiguration['mq']['username'],
        password=externalizedMessageQueueConfiguration['mq']['password'],
        visa_library=externalizedVisaConfiguration['assetadapter']['backend'],
        exchange=externalizedMessageQueueConfiguration['mq']['exchange']
    )

    logging.info('VISA RPC SERVER CONFIGURATION BEGIN ===============================')
    logging.info(externalizedMessageQueueConfiguration)
    logging.info(externalizedVisaConfiguration)
    logging.info('VISA RPC SERVER CONFIGURATION END =================================')

    try:
        server.start()
    except KeyboardInterrupt:
        logging.info('Closed by user.')

    server.stop()


import json
import logging

import rpc.rpc as rpc


class VisaRpcClient(rpc.RpcClient):
    def __init__(self, username, password, name='query', exchange='visa_rpc', host='localhost', port=5432):
        rpc.RpcClient.__init__(self, host=host, name=name, exchange=exchange, username=username, password=password, port=port)
        self.name = name
        self.declare_method(self.on_response)

    def alias(self, request):
        logging.info('REMOTE.alias request ' + json.dumps(obj=request, ensure_ascii=False))
        return self.call(body=request, method_name='alias')

    def query(self, request):
        logging.info('REMOTE.query request ' + json.dumps(obj=request, ensure_ascii=False))
        return self.call(body=request, method_name='query')

    def read(self, request):
        logging.info('REMOTE.read request ' + json.dumps(obj=request, ensure_ascii=False))
        return self.call(body=request, method_name='read')

    def write(self, request):
        logging.info('REMOTE.write request ' + json.dumps(obj=request, ensure_ascii=False))
        return self.call(body=request, method_name='write')

    def list(self, request):
        logging.info('REMOTE.list request ' + json.dumps(obj=request, ensure_ascii=False))
        return self.call(body=request, method_name='list')



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-11 21:15
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('visaweb', '0008_auto_20170311_1724'),
    ]

    operations = [
        migrations.AlterField(
            model_name='visadevice',
            name='alias',
            field=models.CharField(default='248eda5b-68f5-4b0c-ace2-fd9d23bada4e', max_length=200),
        ),
    ]



from django.http import Http404, HttpResponseRedirect, HttpResponse
from django.shortcuts import get_object_or_404, render
from django.views import generic
from .models import VisaDevice, VisaEvent
import json
import logging

# Create your views here.

from django.urls import reverse
import platform
import netifaces as ni

import rpc.client as client
import visualization.visualizer as vis
import logging

configuration = {
    'mq': {
        'host': 'box.cherubits.hu',
        # 'host': 'localhost',
        'port': 5672,
        'username': 'hedgehog',
        # 'username': 'guest',
        'password': 'qwe123',
        # 'password': 'guest',
        'exchange': 'visa_rpc'
    }
}

rpcClient = client.VisaRpcClient(
    exchange=configuration['mq']['exchange'],
    host=configuration['mq']['host'],
    port=configuration['mq']['port'],
    username=configuration['mq']['username'],
    password=configuration['mq']['password'])

logging.info('VISA client created.')


class IndexView(generic.ListView):
    template_name = 'visaweb/index.html'
    context_object_name = 'device_list'

    def get_queryset(self):
        """Return the last five published questions."""
        return VisaDevice.objects.all()

    def get_context_data(self, **kwargs):
        context = super(IndexView, self).get_context_data(**kwargs)
        context['hostname'] = platform.node()
        nics = []
        for interface_name in ni.interfaces():
            nics.append({
                'name': interface_name,
                'details': ni.ifaddresses(interface_name)
            })
        context['network_interfaces'] = nics
        context['visa_connections'] = rpcClient.list({})
        return context

class DetailView(generic.DetailView):
    model = VisaDevice
    context_object_name = 'device'
    template_name = 'visaweb/detail.html'
    slug_field = 'alias'

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        context['log'] = self.get_object().log()
        return context

    # def get_object(self):
    #     return get_object_or_404(VisaDevice, alias=request.session['device_alias'])

class ResultsView(generic.DetailView):
    model = VisaEvent
    context_object_name = 'action'
    template_name = 'visaweb/results.html'


    def get_context_data(self, **kwargs):
        context = super(ResultsView, self).get_context_data(**kwargs)
        context['device'] = self.get_object().session

        heatmapVisualizer = vis.Heatmap(properties={
            'title': 'Custom SVG',
            'width': 640,
            'height': 480,
            'frequency_span': 40,
            'frequency_center': 100,
            'amplitude_reference': 0,
            'amplitude_span': -100
        })
        context['heatmap'] = heatmapVisualizer.show(json.loads(self.get_object().response))

        spectrumVisualizer = vis.Spectrum(properties={
            'title': 'Custom SVG',
            'width': 640,
            'height': 480,
            'frequency_span': 40,
            'frequency_center': 100,
            'amplitude_reference': 0,
            'amplitude_span': -100
        })
        context['spectrum'] = spectrumVisualizer.show(json.loads(self.get_object().response))

        return context



# def index(request):
#     hostname = platform.node()
#     device_list = VisaDevice.objects.all()
#
#     nics = []
#     for interface_name in ni.interfaces():
#         nics.append({
#             'name': interface_name,
#             'details': ni.ifaddresses(interface_name)
#         })
#
#     context = {
#         'device_list': device_list,
#         'hostname': hostname,
#         'network_interfaces': nics
#     }
#     return render(request, 'visaweb/index.html', context)
#
#
# def detail(request, device_alias):
#     try:
#         device = VisaDevice.objects.get(alias=device_alias)
#         log = device.log()
#     except VisaDevice.DoesNotExist:
#         raise Http404("Device does not exist")
#     return render(request, 'visaweb/detail.html', { 'device': device, 'log': log })

def spectrum(request, device_alias):
    # rheatmapVisualizer = vis.Heatmap(properties={
    #     'title': 'Custom SVG',
    #     'width': 640,
    #     'height': 480,
    #     'frequency_span': 40,
    #     'frequency_center': 100,
    #     'amplitude_reference': 0,
    #     'amplitude_span': -100
    # })
    #
    # dataResponse = heatmapVisualizer.show(visaResponse)

    return HttpResponse('sss')

def waterfall(request, device_alias):
    return HttpResponse("Waterfall of %s." % device_alias)

# def results(request, device_alias, event_id):
#     device = get_object_or_404(VisaDevice, alias=device_alias)
#     event = get_object_or_404(VisaEvent, pk=event_id)
#     return render(request, 'visaweb/results.html', {'device': device, 'action': event})

def attach(request):
    # device = get_object_or_404(VisaDevice, alias=device_alias)
    address = request.POST['address']
    alias = request.POST['alias']

    device = VisaDevice(active=True, connected=True, alias=alias, address=address)
    device.save()

    aliasRequest = {}
    aliasRequest[alias] = address
    rpcClient.alias(aliasRequest)

    # device.alias = address
    # device.address = address
    # device.connected = True
    # device.save()
    return HttpResponseRedirect(reverse('visaweb:index'))

def read(request, device_alias):
    device = get_object_or_404(VisaDevice, alias=device_alias)
    log = device.log()
    try:
        visaRequest = request.POST['command']
        logging.info('VISA: %s read %s' % (device_alias, visaRequest))

    except (KeyError, VisaDevice.DoesNotExist):
        # Redisplay the question voting form.
        return render(request, 'visaweb/detail.html', {
            'device': device,
            'log': log,
            'error_message': "Wrong VISA message."
        })
    else:
        try:
            aliasRequest = {}
            aliasRequest['agilent_hp_8920B'] = 'GPIB0::8::INSTR'
            rpcClient.alias(aliasRequest)

            visaResponse = rpcClient.query({
                'alias': device_alias,
                'command': visaRequest
            })
        except (Exception):
            e = VisaDevice.objects.get(alias=device_alias).read(
                request=visaRequest,
                response=json.dumps(visaResponse),
                success=False)
        else:
            e = VisaDevice.objects.get(alias=device_alias).read(
                request=visaRequest,
                response=json.dumps(visaResponse),
                success=True)

        return HttpResponseRedirect(reverse('visaweb:results', args=(e.id,)))

def write(request, device_alias):
    device = get_object_or_404(VisaDevice, alias=device_alias)
    log = device.log()
    try:
        visaRequest = request.POST['command']
        logging.info('VISA: %s write %s' % (device_alias, visaRequest))
    except (KeyError, VisaDevice.DoesNotExist):
        # Redisplay the question voting form.
        return render(request, 'visaweb/detail.html', {
            'device': device,
            'log': log,
            'error_message': "Wrong VISA message."
        })
    else:
        try:
            aliasRequest = {}
            aliasRequest['agilent_hp_8920B'] = 'GPIB0::8::INSTR'
            rpcClient.alias(aliasRequest)

            visaResponse = rpcClient.query({
                'alias': device_alias,
                'command': visaRequest
            })
        except (Exception):
            e = VisaDevice.objects.get(alias=device_alias).write(
                request=visaRequest,
                response=json.dumps(visaResponse),
                success=False)
        else:
            e = VisaDevice.objects.get(alias=device_alias).write(
                request=visaRequest,
                response=json.dumps(visaResponse),
                success=True)

        return HttpResponseRedirect(reverse('visaweb:results', args=(e.id,)))

def query(request, device_alias):
    device = get_object_or_404(VisaDevice, alias=device_alias)
    log = device.log()
    try:
        visaRequest = request.POST['command']
        logging.info('VISA: %s query %s' % (device_alias, visaRequest))


    except (KeyError, VisaDevice.DoesNotExist):
        # Redisplay the question voting form.
        return render(request, 'visaweb/detail.html', {
            'device': device,
            'log': log,
            'error_message': "Wrong VISA message."
        })
    else:
        try:
            aliasRequest = {}
            aliasRequest['agilent_hp_8920B'] = 'GPIB0::8::INSTR'
            rpcClient.alias(aliasRequest)

            visaResponse = rpcClient.query({
                'alias': device_alias,
                'command': visaRequest
            })
            e = VisaDevice.objects.get(alias=device_alias).query(
                request=visaRequest,
                response=json.dumps(visaResponse),
                success=True)
        except (Exception):
            e = VisaDevice.objects.get(alias=device_alias).query(
                request=visaRequest,
                response=json.dumps(visaResponse),
                success=False)

        return HttpResponseRedirect(reverse('visaweb:results', args=(e.id,)))


from _ast import alias

import numpy
import pyvisa as visa
import logging
import datetime

class ScVisaError(Exception):
    def __init__(self, message):
        self.ts = datetime.datetime.now()
        self.message = message
    def __str__(self):
        return '%s: %s' % (self.ts, self.message)

class VisaCommandExecutionError(Exception):
    def __init__(self, alias, address, request, response):
        self.request = request
        self.response = response
        self.address = address
        self.alias = alias
        self.ts = datetime.datetime.now()

    def __str__(self):
        return '%s[%s]: %s -> %s' % (self.alias, self.address, self.request, self.response)

class ResponseBuilder:

    def __init__(self, response={}):
        self.message = response
        self.results = [],
        self.errors = []

    def alias(self, count):
        self.results.append({
            'connectedDevices': count
        })

    def success(self, ts, request, response):
        self.results.append({
            'ts': ts,
            'request': request,
            'response': response
        })
        return self

    def error(self, errorcode, message):
        self.errors.append({
            'error': errorcode,
            'message': message
        })
        return self

    def build(self):
        self.message.results = self.results
        self.message.errors = self.errors
        return self.message

class VisaDeviceManager():
    def __init__(self, visa_library):
        self.rm = visa.ResourceManager(visa_library)
        try:
            logging.info(self.rm.list_resources())
            self.devices = {}
        except Exception as ex:
            logging.exception("message")
            raise ScVisaError(str(ex))

    def alias(self, request):
        index = len(request)
        for key, value in request.items():
            try:
                # res = self.rm.open_resource(value, read_termination='\n')
                res = self.rm.open_resource(value)
                self.devices[key] = res
                index += 1
            except Exception as ex:
                logging.exception("message")
        return {
            'connectedDevices': index
        }

    def query(self, request):
        try:
            logging.info("dm.query" + str(request))
            command = request['command']
            resource = self.devices[request['alias']]
            response = {
                'result': resource.query(command)
            }
            return response
        except Exception as ex:
            logging.exception("message")
            raise ScVisaError(str(ex))

    def visa_query(self, alias, command):
        try:
            logging.info("VISA query on " + alias + ": " + command)
            resource = self.devices[alias]
            # return resource.query_ascii_values(command, container=numpy.array, separator=',', delay=1)
            # return resource.query_ascii_values(command, container=[], separator=',', delay=1)
        except Exception as ex:
            logging.exception("message")
            raise ScVisaError(str(ex))

    def read(self, request):
        try:
            logging.info("dm.read", request)
            command = request['command']
            resource = self.devices[request['alias']]
            response = {
                'result': resource.read(command)
            }
            return response
        except Exception as ex:
            logging.exception("message")
            raise ScVisaError(str(ex))

    def write(self, request):
        try:
            logging.info("dm.write", request)
            command = request['command']
            resource = self.devices[request['alias']]
            resource.write(command)
        except Exception as ex:
            logging.exception("message")
            raise ScVisaError(str(ex))

    def list(self, request):
        try:
            logging.info("dm.list", request)
            return self.rm.list_resources()
        except Exception as ex:
            logging.exception("message")
            raise ScVisaError(str(ex))

    # def query(self, alias, command):
    #     try:
    #         resource = self.devices[alias]
    #         response = resource.query(command)
    #         logging.info("visa query on resource %: %s", alias, command)
    #         return response
    #     except Exception as ex:
    #         logging.exception("message")
    #         raise VisaCommandExecutionError(alias=alias, address=resource._resource_name, request=command, response=ex)

    # def execute(self, alias, command):
    #     builder = ResponseBuilder()
    #
    #     try:
    #         resource = self.devices[alias]
    #     except Exception as ex:
    #         logging.exception("message")
    #         raise VisaCommandExecutionError(alias=alias, address=resource._resource_name, request=command, response=ex)
    #
    #     for command in command_sequence:
    #         try:
    #             result = resource.query(command)
    #             logging.info("visa query on resource %: %s", alias, command)
    #             builder.success()
    #         except Exception as e:
    #             logging.warning("visa query on resource %: %s", alias, command)
    #             builder.error(errorcode=0, message=str(e))
    #
    #     return builder.build()


# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-11 15:46
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='StationControllerSession',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('data_collector', models.CharField(max_length=200)),
                ('start_date', models.DateTimeField()),
                ('end_date', models.DateTimeField()),
            ],
        ),
        migrations.CreateModel(
            name='VisaEvent',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('alias', models.CharField(max_length=200)),
                ('address', models.CharField(max_length=200)),
                ('command', models.CharField(max_length=200)),
                ('execution_date', models.DateTimeField(verbose_name='date published')),
                ('session', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='visaweb.StationControllerSession')),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-11 16:26
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('visaweb', '0002_auto_20170311_1614'),
    ]

    operations = [
        migrations.AddField(
            model_name='visaevent',
            name='operation',
            field=models.CharField(default='query', max_length=200),
        ),
        migrations.AddField(
            model_name='visaevent',
            name='success',
            field=models.BooleanField(default=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-11 16:14
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('visaweb', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='VisaSession',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('alias', models.CharField(max_length=200)),
                ('address', models.CharField(max_length=200)),
            ],
        ),
        migrations.RenameField(
            model_name='visaevent',
            old_name='address',
            new_name='request',
        ),
        migrations.RenameField(
            model_name='visaevent',
            old_name='alias',
            new_name='response',
        ),
        migrations.RemoveField(
            model_name='visaevent',
            name='command',
        ),
        migrations.AlterField(
            model_name='visaevent',
            name='session',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='visaweb.VisaSession'),
        ),
        migrations.DeleteModel(
            name='StationControllerSession',
        ),
    ]



from django.test import TestCase
import datetime
from django.utils import timezone

# Create your tests here.

from .models import VisaDevice, VisaEvent


class VisaDeviceMethodTests(TestCase):

    def test_was_published_recently_with_future_question(self):
        """
        was_published_recently() should return False for questions whose
        pub_date is in the future.
        """
        time = timezone.now() + datetime.timedelta(days=30)

        future_question = VisaDevice(alias="testalias", address="testadress", active=True, connected=True)
        # self.assertIs(future_question.query("testrequest", "testresponse", True).success, True)


from django.db import models
import uuid
from django.utils import timezone

# Create your models here.

class VisaDevice(models.Model):
    active = models.BooleanField(default=True)
    connected = models.BooleanField(default=False)
    alias = models.CharField(max_length=200, default=str(uuid.uuid4()))
    address = models.CharField(max_length=200)

    def log(self):
        return self.visaevent_set.all()

    def query(self, request, response, success):
        r = self.visaevent_set.create(request=request, response=response, execution_date=timezone.now(), success=success, operation='query')
        r.save()
        return r

    def read(self, request, response, success):
        r = self.visaevent_set.create(request=request, response=response, execution_date=timezone.now(), success=success, operation='read')
        r.save()
        return r

    def write(self, request, response, success):
        r = self.visaevent_set.create(request=request, response=response, execution_date=timezone.now(), success=success, operation='write')
        r.save()
        return r

class VisaEvent(models.Model):
    session = models.ForeignKey(VisaDevice, on_delete=models.CASCADE)
    request = models.CharField(max_length=200, default="?IDN")
    response = models.CharField(max_length=200)
    execution_date = models.DateTimeField('date published')
    operation = models.CharField(max_length=200, default="query")
    success = models.BooleanField(default=False)

